<?php
namespace Fiscal\Repositories;

class Impostonf extends \Doctrine\ORM\EntityRepository
{

    /**
     * Verifica os impostos obrigatorios da NF. Se este metodo retornar False, e sinal de que existem impostos para
     * determinado tipo de operacao que nao foram calculados.
     *
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $cdimposto
     * @return bool
     */
    public function existeImpostoPorNF($cdfilial, $nrlanctonf, $cdimposto)
    {
        $sql = "SELECT MAX(CDFILIAL)
                 FROM IMPOSTONF
                 WHERE CDFILIAL = $cdfilial
                   AND NRLANCTONF = $nrlanctonf
                   AND CDIMPOSTO = $cdimposto";

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchColumn($sql);
        return (bool) $result;
    }

    public function consultaBloqueio($cdfilial, $nrlanctonf, $nrorg)
    {
        $sql = <<<SQL
    SELECT MAX(I.NRSEQUMAPAPU) AS NRSEQUMAPAPU, MAX(NRSEQMAPOIMP) AS NRSEQMAPOIMP,
           MAX(MP.NMMAPAAPUR)  AS NMMAPAAPUR
      FROM IMPOSTONF I, MAPAAPURAC MP, IMPOSTO II
     WHERE I.CDFILIAL      = :CDFILIAL
       AND I.NRLANCTONF    = :NRLANCTONF
       AND I.NRORG         = :NRORG
       AND ((I.NRSEQUMAPAPU  = MP.NRSEQUMAPAPU) OR (MP.NRSEQUMAPAPU = I.NRSEQMAPOIMP))
       AND II.CDIMPOSTONFE = MP.CDIMPOSTO
       AND MP.IDBLOQMAPAPU = 'S'
SQL;
        try{

            $params = array(
                "CDFILIAL" => $cdfilial,
                "NRLANCTONF" => $nrlanctonf,
                "NRORG" => $nrorg
            );
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * Verifica se existe imposto retido na fonte
     * @param $filial
     * @param $lancamento
     * @param $organizacao
     * @return array
     */
    public static function verificaImpostoRetidoFonte($filial, $lancamento, $organizacao){
        $sql = <<<SQL
        SELECT I.CDIMPOSTO
        FROM IMPOSTONF NF, IMPOSTO I
        WHERE NF.CDFILIAL   = :P_CDFILIAL   AND
              NF.NRLANCTONF = :P_NRLANCTONF AND
              NF.NRORG      = :P_NRORG      AND
              NF.CDIMPOSTO  = I.CDIMPOSTO   AND
              NF.NRORG      = I.NRORG       AND
              I.IDTRATIMPO  = 'R'
SQL;

        $params = array(
            'P_CDFILIAL'      => $filial,
            'P_NRLANCTONF'   => $lancamento,
            'P_NRORG'      => $organizacao
        );

        return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
    }
}