var Est91000_ProgramacaoDoInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, EstUtil, ZHPromise, ZhFilterPreferences, Lib00206_InfComplProduto_Controller, Est91000_ProgramacaoDoInventarioCego_Services) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE;

    //Inventory Widget Functions
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est91000_ProgramacaoDoInventarioCego_Services.getFilialLogada().then(success => {
            const dtInicial = getLocalVar('DTPROCESSA');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];
            rowsToSet.CDFILIAL = [success.dataset.filial[0].CDFILIAL];
            rowsToSet.IDSITESTCEGO = "PR";

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').fields.map(field => {
                    if (field.name === "NMFILIAL" || field.name === "DTCONTAGEM" || field.name === "IDSITESTCEGO") field.reload();
                });
            });
        });
    };

    this.setFilterParams = function(widget, preference = null) {
        let filtro = [];
        const filterRow = preference.FILTER_ROW;

        filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'inventoryGrid', true)
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.end : null
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": filterRow.IDSITESTCEGO ? filterRow.IDSITESTCEGO : null
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": getLocalVar("CDOPERADOR")
            }
        ];

        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.onValidateCheckedRowsHandler = function(widget, mode) {
        const checkedRowsValidated = widget.getCheckedRows().filter(row => row.IDSITESTCEGO !== 'PR');

        if (checkedRowsValidated.length === 0) {
            this.changeHideInconLabelVisible(widget.actions, "Excluir", false, "deleteInventory", true);
        }
        else {
            if (mode === 'select' || mode === 'selectAll') {
                widget.unSelectAll(checkedRowsValidated);
                MessageService.showMessageByCode("BLOCK_CHECKED_ROWS");
            }
        }
    };

    this.onGenerateInventory = function(widget) {
        const inventoryGrid = templateManager.container.getWidget('inventoryGrid');
        const currentRow = widget.currentRow;

        if (!widget.isValid()) return;

        this.checkInventoryHandler(currentRow).then(checkedFiliais => {
            ScreenService.showLoader();
            currentRow.CDOPERADOR = getLocalVar("CDOPERADOR");
            Est91000_ProgramacaoDoInventarioCego_Services.onGenerateInventory(currentRow, checkedFiliais).then(success => {
                inventoryGrid.reload().then(() => {
                    ScreenService.closePopup();
                    ScreenService.hideLoader();
                    inventoryGrid.activate();
                });
            }).catch(error => {
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            });
        });
    };

    this.checkInventoryHandler = (currentRow) => new Promise((resolve, reject) => {
        let checkedFiliais = [],
            confirmationFiliais = [];
        let hasHolidays = false;

        const validatingFilials = () => new Promise((resolve, reject) => {
            let count = 1;
            currentRow.CDFILIAL.forEach(filial => {
                Est91000_ProgramacaoDoInventarioCego_Services.checkInventoryHandler(filial, currentRow.DTCONTAGEM).then((success) => {
                    switch (success.dataset.filialParams.programado) {
                        case "B":
                            hasHolidays = false;
                            confirmationFiliais = [];
                            checkedFiliais = [];
                            return reject('BLOCK_INVENT_EM_ANDAMENTO');
                        case "P":
                            confirmationFiliais.push(filial);
                            break;
                        case "N":
                            if (success.dataset.filialParams.feriado == "S") hasHolidays = true;
                            else checkedFiliais.push(filial);
                            break;
                        default:
                            break;
                    }
                    if (count === currentRow.CDFILIAL.length) return resolve(true);
                    else count++;
                });
            });
        });

        const promiseConfirmMessage = () => new Promise(resolve => {
            MessageService.confirmMessageByCode("UNIDADE_INVENT_PROGRAMADO_CONFIRMATION").then(() => {
                let count = 1;
                confirmationFiliais.forEach(filial => {
                    Est91000_ProgramacaoDoInventarioCego_Services.checkInventoryHandler(filial, currentRow.DTCONTAGEM, false).then((success) => {
                        if (success.dataset.filialParams.feriado == "S") hasHolidays = true;
                        else checkedFiliais.push(filial);
                        if (count === currentRow.CDFILIAL.length) return resolve(true);
                        else count++;
                    });
                });
            });
        });

        validatingFilials().then(() => {
            if (confirmationFiliais.length != 0) {
                promiseConfirmMessage().then(() => {
                    if (hasHolidays === true) MessageService.showMessageByCode('WARNING_GERA_INVENT_FERIADO');
                    return resolve(checkedFiliais);
                });
            }
            else {
                if (hasHolidays === true) MessageService.showMessageByCode('WARNING_GERA_INVENT_FERIADO');
                return resolve(checkedFiliais);
            }
        }).catch(error => {
            MessageService.showMessageByCode(error);
        });
    });

    this.onDeleteIventory = function(widget) {
        widget.dataSource.name = 'est_estoqueCego#/crud/inventestcego';

        if (widget.mode == 'view') {
            General.deleteCrud(widget).then(() => {
                widget.dataSource.name = 'est_estoqueCego#/inventarioCego';
                widget.reload();
            });
        }
        else {
            General.deleteCrudCheckedRows(widget).then(() => {
                widget.dataSource.name = 'est_estoqueCego#/inventarioCego';
                setTimeout(() => {
                    widget.reload();
                }, 200);
            });
        }
    };

    this.onOpenPopUpScheduleInventory = function() {
        const PopUpScheduleInventory = templateManager.container.getWidget('scheduleInventoryPopUp');
        ScreenService.openPopup(PopUpScheduleInventory);
    };

    this.onClosePopUpScheduleInventory = function() {
        const widgetInventory = templateManager.container.getWidget('inventoryGrid');
        ScreenService.closePopup();
        widgetInventory.activate();
        widgetInventory.reload();
    };

    this.inventoryActionsControl = function(widget, eventType) {
        switch (eventType) {
            case 'onActivate':
                if (widget.currentRow.IDSITESTCEGO !== "PR")
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteInventory", false);
                else
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteInventory", true);
                break;
            default:
                return;
        }
    };

    //Product Widget Functions
    this.onAddProducts = function(widget) {
        const widgetInventory = widget.getParent();
        widget.newRow();
        widget.currentRow.CDFILIAL = widgetInventory.currentRow.CDFILIAL;
        widget.currentRow.CDFILIMOVI = widgetInventory.currentRow.CDFILIAL;
        widget.currentRow.CDALMOXARIFE = widgetInventory.currentRow.CDALMOXARIFE;
        widget.currentRow.CDLOCALESTOQ = widgetInventory.currentRow.CDLOCALESTOQ;
        widget.currentRow.DTCONTAGEM = widgetInventory.currentRow.DTCONTAGEM;
    };

    this.onSaveProducts = function(widget) {

        if (widget.currentRow.__is_new == true) {
            widget.currentRow.IDGERITINVCEGPR = "A";
            Est91000_ProgramacaoDoInventarioCego_Services.saveProducts(widget.currentRow).then((saved) => {
                widget.reload().then(() => {
                    ScreenService.goBack();
                    widget.list();
                });

            }).catch(
                error => {
                    if (error.data)
                        ScreenService.showMessage(error.data.error);
                    else
                        ScreenService.showMessage(error);
                }
            );
        }
    };

    this.onDeleteProducts = function(widget) {
        return widget.mode == 'view' ? General.deleteCrud(widget) : General.deleteCrudCheckedRows(widget);
    };

    this.productsActionsControl = function(widget, eventType) {
        switch (eventType) {
            case 'onEnter':
                if (widget.getParent().currentRow.IDSITESTCEGO !== "PR")
                    this.changeHideInconLabelVisible(widget.actions, "Adicionar", true, "Novo", false);
                else
                    this.changeHideInconLabelVisible(widget.actions, "Adicionar", true, "Novo", true);
                break;
            case 'onActivate':
                if (widget.getParent().currentRow.IDSITESTCEGO !== "PR")
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteProducts", false);
                else
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteProducts", true);

                break;
            default:
                return;
        }
    };

    //Support Functions

    //This opens swipe details without change widget mode to view
    this.onOpenDetail = function(widget) {

        Est91000_ProgramacaoDoInventarioCego_Services.getParamFilialData().then((filialParams) => {
            if (filialParams.dataset.paramfilial.length != 0) {

                // Set Global variables
                IDUTILALMOX = filialParams.dataset.paramfilial[0].IDUTILALMOX;
                IDUTILLCESTQ = filialParams.dataset.paramfilial[0].IDUTILLCESTQ;
                IDUTILLOTE = filialParams.dataset.paramfilial[0].IDUTILLOTE;

                if (IDUTILLOTE == 'S') {
                    General.setVisibleField('NRLOTEESTQ', widget.name);
                    General.setVisibleField('NRSUBLOTE', widget.name);
                }
                else {
                    General.setInvisibleField('NRLOTEESTQ', widget.name);
                    General.setInvisibleField('NRSUBLOTE', widget.name);
                }
                widget.redraw();
            }
        });
    };

    this.grayColorHandler = function(row) {
        if(typeof(row.fields) !== 'undefined')return;
        let color = false;
        if (row.IDSITESTCEGO === "PN" || row.IDSITESTCEGO === "DN" || row.IDSITESTCEGO === "EN" ||
            Est91000_ProgramacaoDoInventarioCego_Services.getDate(row.DTCONTAGEM) <
            Est91000_ProgramacaoDoInventarioCego_Services.getDate(row.DTPROCESSA)) color = true;
        return color;
    };

    this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
        data.forEach(element => {
            if (element.name === name) {
                element.label = label;
                element.hideIcon = hide;
                element.isVisible = visibility;
            }
        });
    };

    this.popUpFieldsHandler = function(field) {
        let widget = field.widget;
        let fieldProdIni = widget.getField('NMPRODINI');
        let fieldProdFim = widget.getField('NMPRODFIN');
        let fieldGrupProd = widget.getField('NMGRUPPROD');

        switch (field.name) {
            case 'NMPRODFIN':
            case 'NMPRODINI':
                fieldProdIni.validations = { "required": true };
                fieldProdFim.validations = { "required": true };
                delete fieldGrupProd.validations;
                widget.currentRow.NMGRUPPROD = '';
                break;
            case 'NMGRUPPROD':
                widget.currentRow.NMPRODINI = '';
                widget.currentRow.NMPRODFIN = '';
                delete fieldProdIni.validations;
                delete fieldProdFim.validations;
                fieldGrupProd.validations = { "required": true };
                if (widget.currentRow.NMGRUPPROD === '') delete fieldGrupProd.validations;
                break;
            default:
                return;
        }
    };

    this.checkProductRange = function(field) {
        let currentRow = field.widget.currentRow;

        if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                MessageService.showMessageByCode("PRODUTO_INICIAL_MAIOR_FINAL");
            }
        }
    };

    this.checkDatePopUp = function(field) {
        const currentRow = field.widget.currentRow;
        const todayString = moment().format('DD/MM/YYYY');
        const todayDate = Est91000_ProgramacaoDoInventarioCego_Services.getDate(todayString);
        const dtContagemDate = Est91000_ProgramacaoDoInventarioCego_Services.getDate(currentRow.DTCONTAGEM);

        if (dtContagemDate < todayDate && dtContagemDate != '') currentRow.DTCONTAGEM = todayString;
    };

    this.fieldsFilterPopUpControl = function(field) {
        const popUpWidget = templateManager.container.getWidget("scheduleInventoryPopUp");
        const prodIniField = popUpWidget.getField('NMPRODINI');
        const prodFinField = popUpWidget.getField('NMPRODFIN');
        const groupField = popUpWidget.getField('NMGRUPPROD');

        switch (field.name) {
            case 'NMGRUPPROD':
            case 'NMPRODINI':
            case 'NMPRODFIN':
                if ((popUpWidget.currentRow.NMGRUPPROD == null || popUpWidget.currentRow.NMGRUPPROD == "") &&
                    (popUpWidget.currentRow.NMPRODINI == null || popUpWidget.currentRow.NMPRODINI == "") &&
                    (popUpWidget.currentRow.NMPRODFIN == null || popUpWidget.currentRow.NMPRODFIN == "")) {
                    groupField.validations.required = true;
                }
                else {
                    delete groupField.validations.required;
                }
                break;
            default:
                break;
        }
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est91000_ProgramacaoDoInventarioCego_Controller', Est91000_ProgramacaoDoInventarioCego_Controller);
});
